@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -250,7 +250,9 @@ class IReactorTCP(Interface):
 
         @param backlog: size of the listen queue
 
-        @param interface: the hostname to bind to, defaults to '' (all)
+        @param interface: The local IPv4 or IPv6 address to which to bind;
+            defaults to '', ie all IPv4 addresses.  To bind to all IPv4 and IPv6
+            addresses, you must call this method twice.
 
         @return: an object that provides L{IListeningPort}.
 
@@ -492,6 +494,73 @@ class IReactorMulticast(Interface):
 
 
 
+class IReactorSocket(Interface):
+    """
+    Methods which allow a reactor to use externally created sockets.
+
+    For example, to use C{adoptStreamPort} to implement behavior equivalent
+    to that of L{IReactorTCP.listenTCP}, you might write code like this::
+
+        from socket import SOMAXCONN, AF_INET, SOCK_STREAM, socket
+        portSocket = socket(AF_INET, SOCK_STREAM)
+        # Set FD_CLOEXEC on port, left as an exercise.  Then make it into a
+        # non-blocking listening port:
+        portSocket.setblocking(False)
+        portSocket.bind(('192.168.1.2', 12345))
+        portSocket.listen(SOMAXCONN)
+
+        # Now have the reactor use it as a TCP port
+        port = reactor.adoptStreamPort(
+            portSocket.fileno(), AF_INET, YourFactory())
+
+        # portSocket itself is no longer necessary, and needs to be cleaned
+        # up by us.
+        portSocket.close()
+
+        # Whenever the server is no longer needed, stop it as usual.
+        stoppedDeferred = port.stopListening()
+
+    Another potential use is to inherit a listening descriptor from a parent
+    process (for example, systemd or launchd), or to receive one over a UNIX
+    domain socket.
+
+    Some plans for extending this interface exist.  See:
+
+        - U{http://twistedmatrix.com/trac/ticket/5570}: established connections
+        - U{http://twistedmatrix.com/trac/ticket/5573}: AF_UNIX ports
+        - U{http://twistedmatrix.com/trac/ticket/5574}: SOCK_DGRAM sockets
+    """
+
+    def adoptStreamPort(fileDescriptor, addressFamily, factory):
+        """
+        Add an existing listening I{SOCK_STREAM} socket to the reactor to
+        monitor for new connections to accept and handle.
+        @param fileDescriptor: A file descriptor associated with a socket which
+            is already bound to an address and marked as listening.  The socket
+            must be set non-blocking.  Any additional flags (for example,
+            close-on-exec) must also be set by application code.  Application
+            code is responsible for closing the file descriptor, which may be
+            done as soon as C{adoptStreamPort} returns.
+        @type fileDescriptor: C{int}
+
+        @param addressFamily: The address family (or I{domain}) of the socket.
+            For example, L{socket.AF_INET6}.
+
+        @param factory: A L{ServerFactory} instance to use to create new
+            protocols to handle connections accepted via this socket.
+
+        @return: An object providing L{IListeningPort}.
+
+        @raise UnsupportedAddressFamily: If the given address family is not
+            supported by this reactor, or not supported with the given socket
+            type.
+
+        @raise UnsupportedSocketType: If the given socket type is not supported
+            by this reactor, or not supported with the given socket type.
+        """
+
+
+
 class IReactorProcess(Interface):
 
     def spawnProcess(processProtocol, executable, args=(), env={}, path=None,
@@ -870,6 +939,37 @@ class IReactorPluggableResolver(Interface):
         """
 
 
+class IReactorDaemonize(Interface):
+    """
+    A reactor which provides hooks that need to be called before and after
+    daemonization.
+
+    Notes:
+       - This interface SHOULD NOT be called by applications.
+       - This interface should only be implemented by reactors as a workaround
+         (in particular, it's implemented currently only by kqueue()).
+         For details please see the comments on ticket #1918.
+    """
+
+    def beforeDaemonize():
+        """
+        Hook to be called immediately before daemonization. No reactor methods
+        may be called until L{afterDaemonize} is called.
+
+        @return: C{None}.
+        """
+
+
+    def afterDaemonize():
+        """
+        Hook to be called immediately after daemonization. This may only be
+        called after L{beforeDaemonize} had been called previously.
+
+        @return: C{None}.
+        """
+
+
+
 class IReactorFDSet(Interface):
     """
     Implement me to be able to use L{IFileDescriptor} type resources.
@@ -986,6 +1086,7 @@ class ILoggingContext(Interface):
         """
 
 
+
 class IFileDescriptor(ILoggingContext):
     """
     An interface representing a UNIX-style numeric file descriptor.
@@ -993,10 +1094,17 @@ class IFileDescriptor(ILoggingContext):
 
     def fileno():
         """
+        @raise: If the descriptor no longer has a valid file descriptor
+            number associated with it.
+
         @return: The platform-specified representation of a file descriptor
-                 number.
+            number.  Or C{-1} if the descriptor no longer has a valid file
+            descriptor number associated with it.  As long as the descriptor
+            is valid, calls to this method on a particular instance must
+            return the same value.
         """
 
+
     def connectionLost(reason):
         """
         Called when the connection was lost.
@@ -1016,6 +1124,7 @@ class IFileDescriptor(ILoggingContext):
         """
 
 
+
 class IReadDescriptor(IFileDescriptor):
     """
     An L{IFileDescriptor} that can read.
@@ -1026,6 +1135,10 @@ class IReadDescriptor(IFileDescriptor):
     def doRead():
         """
         Some data is available for reading on your descriptor.
+
+        @return: If an error is encountered which causes the descriptor to
+            no longer be valid, a L{Failure} should be returned.  Otherwise,
+            C{None}.
         """
 
 
@@ -1039,6 +1152,10 @@ class IWriteDescriptor(IFileDescriptor):
     def doWrite():
         """
         Some data can be written to your descriptor.
+
+        @return: If an error is encountered which causes the descriptor to
+            no longer be valid, a L{Failure} should be returned.  Otherwise,
+            C{None}.
         """
 
 
@@ -1114,26 +1231,44 @@ class IConsumer(Interface):
         @return: C{None}
         """
 
+
     def unregisterProducer():
         """
         Stop consuming data from a producer, without disconnecting.
         """
 
+
     def write(data):
         """
         The producer will write data by calling this method.
+
+        The implementation must be non-blocking and perform whatever
+        buffering is necessary.  If the producer has provided enough data
+        for now and it is a L{IPushProducer}, the consumer may call its
+        C{pauseProducing} method.
         """
 
+
+
+deprecatedModuleAttribute(Version("Twisted", 11, 1, 0),
+    "Please use IConsumer (and IConsumer.unregisterProducer) instead.",
+    __name__, "IFinishableConsumer")
+
 class IFinishableConsumer(IConsumer):
     """
-    A Consumer for producers that finish.
+    A Consumer for producers that finish.  This interface offers no advantages
+    over L{IConsumer} and is deprecated.  Please use
+    L{IConsumer.unregisterProducer} instead of L{IFinishableConsumer.finish}.
     """
 
     def finish():
         """
-        The producer has finished producing.
+        The producer has finished producing.  This method is deprecated.
+        Please use L{IConsumer.unregisterProducer} instead.
         """
 
+
+
 class IProducer(Interface):
     """
     A producer produces data for a consumer.
@@ -1154,7 +1289,7 @@ class IProducer(Interface):
 class IPushProducer(IProducer):
     """
     A push producer, also known as a streaming producer is expected to
-    produce (write to this consumer) data on a continous basis, unless
+    produce (write to this consumer) data on a continuous basis, unless
     it has been paused. A paused push producer will resume producing
     after its resumeProducing() method is called.   For a push producer
     which is not pauseable, these functions may be noops.
@@ -1320,7 +1455,7 @@ class IHalfCloseableProtocol(Interface):
         Notification of the read connection being closed.
 
         This indicates peer did half-close of write side. It is now
-        the responsiblity of the this protocol to call
+        the responsibility of the this protocol to call
         loseConnection().  In addition, the protocol MUST make sure a
         reference to it still exists (i.e. by doing a callLater with
         one of its methods, etc.)  as the reactor will only have a
@@ -1339,6 +1474,26 @@ class IHalfCloseableProtocol(Interface):
         """
 
 
+
+class IFileDescriptorReceiver(Interface):
+    """
+    Protocols may implement L{IFileDescriptorReceiver} to receive file
+    descriptors sent to them.  This is useful in conjunction with
+    L{IUNIXTransport}, which allows file descriptors to be sent between
+    processes on a single host.
+    """
+    def fileDescriptorReceived(descriptor):
+        """
+        Called when a file descriptor is received over the connection.
+
+        @param descriptor: The descriptor which was received.
+        @type descriptor: C{int}
+
+        @return: C{None}
+        """
+
+
+
 class IProtocolFactory(Interface):
     """
     Interface for protocol factories.
@@ -1446,6 +1601,19 @@ class ITCPTransport(ITransport):
         producer.
         """
 
+
+    def abortConnection():
+        """
+        Close the connection abruptly.
+
+        Discards any buffered data, stops any registered producer,
+        and, if possible, notifies the other end of the unclean
+        closure.
+
+        @since: 11.1
+        """
+
+
     def getTcpNoDelay():
         """
         Return if C{TCP_NODELAY} is enabled.
@@ -1475,15 +1643,43 @@ class ITCPTransport(ITransport):
 
     def getHost():
         """
-        Returns L{IPv4Address}.
+        Returns L{IPv4Address} or L{IPv6Address}.
         """
 
     def getPeer():
         """
-        Returns L{IPv4Address}.
+        Returns L{IPv4Address} or L{IPv6Address}.
         """
 
 
+
+class IUNIXTransport(ITransport):
+    """
+    Transport for stream-oriented unix domain connections.
+    """
+    def sendFileDescriptor(descriptor):
+        """
+        Send a duplicate of this (file, socket, pipe, etc) descriptor to the
+        other end of this connection.
+
+        The send is non-blocking and will be queued if it cannot be performed
+        immediately.  The send will be processed in order with respect to other
+        C{sendFileDescriptor} calls on this transport, but not necessarily with
+        respect to C{write} calls on this transport.  The send can only be
+        processed if there are also bytes in the normal connection-oriented send
+        buffer (ie, you must call C{write} at least as many times as you call
+        C{sendFileDescriptor}).
+
+        @param descriptor: An C{int} giving a valid file descriptor in this
+            process.  Note that a I{file descriptor} may actually refer to a
+            socket, a pipe, or anything else POSIX tries to treat in the same
+            way as a file.
+
+        @return: C{None}
+        """
+
+
+
 class ITLSTransport(ITCPTransport):
     """
     A TCP transport that supports switching to TLS midstream.
@@ -1546,8 +1742,6 @@ class IProcessTransport(ITransport):
         Similar to L{ITransport.write} but also allows the file descriptor in
         the child process which will receive the bytes to be specified.
 
-        This is not available on all platforms.
-
         @type childFD: C{int}
         @param childFD: The file descriptor to which to write.
 
@@ -1555,6 +1749,10 @@ class IProcessTransport(ITransport):
         @param data: The bytes to write.
 
         @return: C{None}
+
+        @raise KeyError: If C{childFD} is not a file descriptor that was mapped
+            in the child when L{IReactorProcess.spawnProcess} was used to create
+            it.
         """
 
     def loseConnection():
@@ -1829,14 +2027,12 @@ class IStreamClientEndpointStringParser(Interface):
     )
 
 
-    def parseStreamClient(reactor, *args, **kwargs):
+    def parseStreamClient(*args, **kwargs):
         """
         This method is invoked by L{endpoints.clientFromString}, if the type of
         endpoint matches the return value from this
         L{IStreamClientEndpointStringParser}'s C{prefix} method.
 
-        @param reactor: The reactor passed to C{clientFromString}.
-
         @param args: The string arguments, minus the endpoint type, in the
             endpoint description string, parsed according to the rules
             described in L{endpoints.quoteStringArgument}.  For example, if the
@@ -1851,4 +2047,3 @@ class IStreamClientEndpointStringParser(Interface):
         @return: a client endpoint
         @rtype: L{IStreamClientEndpoint}
         """
-
