@@ -10,12 +10,14 @@ import warnings
 import socket
 import errno
 import os
+import sys
 
 from zope.interface import implements, classImplements
 
 from twisted.python.compat import set
 from twisted.internet.interfaces import IReactorUNIX, IReactorUNIXDatagram
-from twisted.internet.interfaces import IReactorTCP, IReactorUDP, IReactorSSL, _IReactorArbitrary
+from twisted.internet.interfaces import (
+    IReactorTCP, IReactorUDP, IReactorSSL, _IReactorArbitrary, IReactorSocket)
 from twisted.internet.interfaces import IReactorProcess, IReactorMulticast
 from twisted.internet.interfaces import IHalfCloseableDescriptor
 from twisted.internet import error
@@ -26,12 +28,21 @@ from twisted.persisted import styles
 from twisted.python.runtime import platformType, platform
 
 from twisted.internet.base import ReactorBase, _SignalReactorMixin
+from twisted.internet.main import CONNECTION_DONE, CONNECTION_LOST
+
+# Exceptions that doSelect might return frequently
+_NO_FILENO = error.ConnectionFdescWentAway('Handler has no fileno method')
+_NO_FILEDESC = error.ConnectionFdescWentAway('File descriptor lost')
+
 
 try:
-    from twisted.internet import ssl
-    sslEnabled = True
+    from twisted.protocols import tls
 except ImportError:
-    sslEnabled = False
+    tls = None
+    try:
+        from twisted.internet import ssl
+    except ImportError:
+        ssl = None
 
 try:
     from twisted.internet import unix
@@ -223,14 +234,11 @@ class _SIGCHLDWaker(_FDWaker):
 
 
 
-class PosixReactorBase(_SignalReactorMixin, ReactorBase):
-    """
-    A basis for reactors that use file descriptors.
 
-    @ivar _childWaker: C{None} or a reference to the L{_SIGCHLDWaker}
-        which is used to properly notice child process termination.
+class _DisconnectSelectableMixin(object):
+    """
+    Mixin providing the C{_disconnectSelectable} method.
     """
-    implements(_IReactorArbitrary, IReactorTCP, IReactorUDP, IReactorMulticast)
 
     def _disconnectSelectable(self, selectable, why, isRead, faildict={
         error.ConnectionDone: failure.Failure(error.ConnectionDone()),
@@ -255,6 +263,22 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
             self.removeWriter(selectable)
             selectable.connectionLost(failure.Failure(why))
 
+
+
+class PosixReactorBase(_SignalReactorMixin, _DisconnectSelectableMixin,
+                       ReactorBase):
+    """
+    A basis for reactors that use file descriptors.
+
+    @ivar _childWaker: C{None} or a reference to the L{_SIGCHLDWaker}
+        which is used to properly notice child process termination.
+    """
+    implements(_IReactorArbitrary, IReactorTCP, IReactorUDP, IReactorMulticast)
+
+    # Callable that creates a waker, overrideable so that subclasses can
+    # substitute their own implementation:
+    _wakerFactory = _Waker
+
     def installWaker(self):
         """
         Install a `waker' to allow threads and signals to wake up the IO thread.
@@ -263,7 +287,7 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
         the reactor. On Windows we use a pair of sockets.
         """
         if not self.waker:
-            self.waker = _Waker(self)
+            self.waker = self._wakerFactory(self)
             self._internalReaders.add(self.waker)
             self.addReader(self.waker)
 
@@ -321,8 +345,10 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
                 return process.Process(self, executable, args, env, path,
                                        processProtocol, uid, gid, childFDs)
         elif platformType == "win32":
-            if uid is not None or gid is not None:
-                raise ValueError("The uid and gid parameters are not supported on Windows.")
+            if uid is not None:
+                raise ValueError("Setting UID is unsupported on this platform.")
+            if gid is not None:
+                raise ValueError("Setting GID is unsupported on this platform.")
             if usePTY:
                 raise ValueError("The usePTY parameter is not supported on Windows.")
             if childFDs:
@@ -410,6 +436,25 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
         return p
 
 
+    # IReactorSocket (but not on Windows)
+    def adoptStreamPort(self, fileDescriptor, addressFamily, factory):
+        """
+        Create a new L{IListeningPort} from an already-initialized socket.
+
+        This just dispatches to a suitable port implementation (eg from
+        L{IReactorTCP}, etc) based on the specified C{addressFamily}.
+
+        @see: L{twisted.internet.interfaces.IReactorSocket.adoptStreamPort}
+        """
+        if addressFamily not in (socket.AF_INET, socket.AF_INET6):
+            raise error.UnsupportedAddressFamily(addressFamily)
+
+        p = tcp.Port._fromListeningDescriptor(
+            self, fileDescriptor, addressFamily, factory)
+        p.startListening()
+        return p
+
+
     # IReactorTCP
 
     def listenTCP(self, port, factory, backlog=50, interface=''):
@@ -431,18 +476,34 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
     def connectSSL(self, host, port, factory, contextFactory, timeout=30, bindAddress=None):
         """@see: twisted.internet.interfaces.IReactorSSL.connectSSL
         """
-        assert sslEnabled, "SSL support is not present"
-        c = ssl.Connector(host, port, factory, contextFactory, timeout, bindAddress, self)
-        c.connect()
-        return c
+        if tls is not None:
+            tlsFactory = tls.TLSMemoryBIOFactory(contextFactory, True, factory)
+            return self.connectTCP(host, port, tlsFactory, timeout, bindAddress)
+        elif ssl is not None:
+            c = ssl.Connector(
+                host, port, factory, contextFactory, timeout, bindAddress, self)
+            c.connect()
+            return c
+        else:
+            assert False, "SSL support is not present"
+
+
 
     def listenSSL(self, port, factory, contextFactory, backlog=50, interface=''):
         """@see: twisted.internet.interfaces.IReactorSSL.listenSSL
         """
-        assert sslEnabled, "SSL support is not present"
-        p = ssl.Port(port, factory, contextFactory, backlog, interface, self)
-        p.startListening()
-        return p
+        if tls is not None:
+            tlsFactory = tls.TLSMemoryBIOFactory(contextFactory, False, factory)
+            port = self.listenTCP(port, tlsFactory, backlog, interface)
+            port._type = 'TLS'
+            return port
+        elif ssl is not None:
+            p = ssl.Port(
+                port, factory, contextFactory, backlog, interface, self)
+            p.startListening()
+            return p
+        else:
+            assert False, "SSL support is not present"
 
 
     # IReactorArbitrary
@@ -493,11 +554,87 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
         return list(removedReaders | removedWriters)
 
 
-if sslEnabled:
+class _PollLikeMixin(object):
+    """
+    Mixin for poll-like reactors.
+
+    Subclasses must define the following attributes::
+
+      - _POLL_DISCONNECTED - Bitmask for events indicating a connection was
+        lost.
+      - _POLL_IN - Bitmask for events indicating there is input to read.
+      - _POLL_OUT - Bitmask for events indicating output can be written.
+
+    Must be mixed in to a subclass of PosixReactorBase (for
+    _disconnectSelectable).
+    """
+
+    def _doReadOrWrite(self, selectable, fd, event):
+        """
+        fd is available for read or write, do the work and raise errors if
+        necessary.
+        """
+        why = None
+        inRead = False
+        if event & self._POLL_DISCONNECTED and not (event & self._POLL_IN):
+            # Handle disconnection.  But only if we finished processing all
+            # the pending input.
+            if fd in self._reads:
+                # If we were reading from the descriptor then this is a
+                # clean shutdown.  We know there are no read events pending
+                # because we just checked above.  It also might be a
+                # half-close (which is why we have to keep track of inRead).
+                inRead = True
+                why = CONNECTION_DONE
+            else:
+                # If we weren't reading, this is an error shutdown of some
+                # sort.
+                why = CONNECTION_LOST
+        else:
+            # Any non-disconnect event turns into a doRead or a doWrite.
+            try:
+                # First check to see if the descriptor is still valid.  This
+                # gives fileno() a chance to raise an exception, too. 
+                # Ideally, disconnection would always be indicated by the
+                # return value of doRead or doWrite (or an exception from
+                # one of those methods), but calling fileno here helps make
+                # buggy applications more transparent.
+                if selectable.fileno() == -1:
+                    # -1 is sort of a historical Python artifact.  Python
+                    # files and sockets used to change their file descriptor
+                    # to -1 when they closed.  For the time being, we'll
+                    # continue to support this anyway in case applications
+                    # replicated it, plus abstract.FileDescriptor.fileno
+                    # returns -1.  Eventually it'd be good to deprecate this
+                    # case.
+                    why = _NO_FILEDESC
+                else:
+                    if event & self._POLL_IN:
+                        # Handle a read event.
+                        why = selectable.doRead()
+                        inRead = True
+                    if not why and event & self._POLL_OUT:
+                        # Handle a write event, as long as doRead didn't
+                        # disconnect us.
+                        why = selectable.doWrite()
+                        inRead = False
+            except:
+                # Any exception from application code gets logged and will
+                # cause us to disconnect the selectable.
+                why = sys.exc_info()[1]
+                log.err()
+        if why:
+            self._disconnectSelectable(selectable, why, inRead)
+
+
+
+if tls is not None or ssl is not None:
     classImplements(PosixReactorBase, IReactorSSL)
 if unixEnabled:
     classImplements(PosixReactorBase, IReactorUNIX, IReactorUNIXDatagram)
 if processEnabled:
     classImplements(PosixReactorBase, IReactorProcess)
+if getattr(socket, 'fromfd', None) is not None:
+    classImplements(PosixReactorBase, IReactorSocket)
 
 __all__ = ["PosixReactorBase"]
