@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.names.test.test_dns -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -29,7 +29,7 @@ __all__ = [
     'Record_DNAME', 'Record_HINFO', 'Record_MB', 'Record_MD', 'Record_MF',
     'Record_MG', 'Record_MINFO', 'Record_MR', 'Record_MX', 'Record_NAPTR',
     'Record_NS', 'Record_NULL', 'Record_PTR', 'Record_RP', 'Record_SOA',
-    'Record_SPF', 'Record_SRV', 'Record_TXT', 'Record_WKS',
+    'Record_SPF', 'Record_SRV', 'Record_TXT', 'Record_WKS', 'UnknownRecord',
 
     'QUERY_CLASSES', 'QUERY_TYPES', 'REV_CLASSES', 'REV_TYPES', 'EXT_QUERIES',
 
@@ -48,10 +48,7 @@ import warnings
 
 import struct, random, types, socket
 
-try:
-    import cStringIO as StringIO
-except ImportError:
-    import StringIO
+import cStringIO as StringIO
 
 AF_INET6 = socket.AF_INET6
 
@@ -332,7 +329,11 @@ class Name:
 
         @raise EOFError: Raised when there are not enough bytes available
         from C{strio}.
+
+        @raise ValueError: Raised when the name cannot be decoded (for example,
+            because it contains a loop).
         """
+        visited = set()
         self.name = ''
         off = 0
         while 1:
@@ -344,6 +345,9 @@ class Name:
             if (l >> 6) == 3:
                 new_off = ((l&63) << 8
                             | ord(readPrecisely(strio, 1)))
+                if new_off in visited:
+                    raise ValueError("Compression loop in encoded name")
+                visited.add(new_off)
                 if off == 0:
                     off = strio.tell()
                 strio.seek(new_off)
@@ -441,7 +445,9 @@ class RRHeader(tputil.FancyEqMixin):
     @ivar cls: The query class of the original request.
     @ivar ttl: The time-to-live for this record.
     @ivar payload: An object that implements the IEncodable interface
-    @ivar auth: Whether this header is authoritative or not.
+
+    @ivar auth: A C{bool} indicating whether this C{RRHeader} was parsed from an
+        authoritative message.
     """
 
     implements(IEncodable)
@@ -476,7 +482,7 @@ class RRHeader(tputil.FancyEqMixin):
         @type payload: An object implementing C{IEncodable}
         @param payload: A Query Type specific data object.
         """
-        assert (payload is None) or (payload.TYPE == type)
+        assert (payload is None) or isinstance(payload, UnknownRecord) or (payload.TYPE == type)
 
         self.name = Name(name)
         self.type = type
@@ -1404,12 +1410,6 @@ class Record_MX(tputil.FancyStrMixin, tputil.FancyEqMixin):
         self.name = Name()
         self.name.decode(strio)
 
-    def exchange(self):
-        warnings.warn("use Record_MX.name instead", DeprecationWarning, stacklevel=2)
-        return self.name
-
-    exchange = property(exchange)
-
     def __hash__(self):
         return hash((self.preference, self.name))
 
@@ -1422,7 +1422,7 @@ class Record_TXT(tputil.FancyEqMixin, tputil.FancyStrMixin):
 
     @type data: C{list} of C{str}
     @ivar data: Freeform text which makes up this record.
-    
+
     @type ttl: C{int}
     @ivar ttl: The maximum number of seconds which this record should be cached.
     """
@@ -1464,14 +1464,62 @@ class Record_TXT(tputil.FancyEqMixin, tputil.FancyStrMixin):
 
 
 
+# This is a fallback record
+class UnknownRecord(tputil.FancyEqMixin, tputil.FancyStrMixin, object):
+    """
+    Encapsulate the wire data for unkown record types so that they can
+    pass through the system unchanged.
+
+    @type data: C{str}
+    @ivar data: Wire data which makes up this record.
+
+    @type ttl: C{int}
+    @ivar ttl: The maximum number of seconds which this record should be cached.
+
+    @since: 11.1
+    """
+    implements(IEncodable, IRecord)
+
+    fancybasename = 'UNKNOWN'
+    compareAttributes = ('data', 'ttl')
+    showAttributes = ('data', 'ttl')
+
+    def __init__(self, data='', ttl=None):
+        self.data = data
+        self.ttl = str2time(ttl)
+
+
+    def encode(self, strio, compDict=None):
+        """
+        Write the raw bytes corresponding to this record's payload to the
+        stream.
+        """
+        strio.write(self.data)
+
+
+    def decode(self, strio, length=None):
+        """
+        Load the bytes which are part of this record from the stream and store
+        them unparsed and unmodified.
+        """
+        if length is None:
+            raise Exception('must know length for unknown record types')
+        self.data = readPrecisely(strio, length)
+
+
+    def __hash__(self):
+        return hash((self.data, self.ttl))
+
+
+
 class Record_SPF(Record_TXT):
     """
     Structurally, freeform text. Semantically, a policy definition, formatted
     as defined in U{rfc 4408<http://www.faqs.org/rfcs/rfc4408.html>}.
-    
+
     @type data: C{list} of C{str}
     @ivar data: Freeform text which makes up this record.
-    
+
     @type ttl: C{int}
     @ivar ttl: The maximum number of seconds which this record should be cached.
     """
@@ -1583,7 +1631,7 @@ class Message:
 
     def parseRecords(self, list, num, strio):
         for i in range(num):
-            header = RRHeader()
+            header = RRHeader(auth=self.auth)
             try:
                 header.decode(strio)
             except EOFError:
@@ -1624,7 +1672,7 @@ class Message:
             can be found for the given type.
         @rtype: L{types.ClassType}
         """
-        return self._recordTypes.get(type, None)
+        return self._recordTypes.get(type, UnknownRecord)
 
 
     def toStr(self):
