@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_paths -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -17,7 +17,11 @@ from os.path import join as joinpath
 from os import sep as slash
 from os import listdir, utime, stat
 
-from stat import S_ISREG, S_ISDIR
+from stat import S_ISREG, S_ISDIR, S_IMODE, S_ISBLK, S_ISSOCK
+from stat import S_IRUSR, S_IWUSR, S_IXUSR
+from stat import S_IRGRP, S_IWGRP, S_IXGRP
+from stat import S_IROTH, S_IWOTH, S_IXOTH
+
 
 # Please keep this as light as possible on other Twisted imports; many, many
 # things import this module, and it would be good if it could easily be
@@ -30,6 +34,10 @@ from twisted.python.win32 import ERROR_FILE_NOT_FOUND, ERROR_PATH_NOT_FOUND
 from twisted.python.win32 import ERROR_INVALID_NAME, ERROR_DIRECTORY, O_BINARY
 from twisted.python.win32 import WindowsError
 
+from twisted.python.util import FancyEqMixin
+
+from zope.interface import Interface, Attribute, implements
+
 _CREATE_FLAGS = (os.O_EXCL |
                  os.O_CREAT |
                  os.O_RDWR |
@@ -73,6 +81,133 @@ islink = getattr(os.path, 'islink', _stub_islink)
 randomBytes = getattr(os, 'urandom', _stub_urandom)
 armor = getattr(base64, 'urlsafe_b64encode', _stub_armor)
 
+class IFilePath(Interface):
+    """
+    File path object.
+
+    A file path represents a location for a file-like-object and can be
+    organized into a hierarchy; a file path can can children which are
+    themselves file paths.
+
+    A file path has a name which unique identifies it in the context of its
+    parent (if it has one); a file path can not have two children with the same
+    name.  This name is referred to as the file path's "base name".
+
+    A series of such names can be used to locate nested children of a file path;
+    such a series is referred to as the child's "path", relative to the parent.
+    In this case, each name in the path is referred to as a "path segment"; the
+    child's base name is the segment in the path.
+
+    When representing a file path as a string, a "path separator" is used to
+    delimit the path segments within the string.  For a file system path, that
+    would be C{os.sep}.
+
+    Note that the values of child names may be restricted.  For example, a file
+    system path will not allow the use of the path separator in a name, and
+    certain names (eg. C{"."} and C{".."}) may be reserved or have special
+    meanings.
+
+    @since: 12.1
+    """
+    sep = Attribute("The path separator to use in string representations")
+
+    def child(name):
+        """
+        Obtain a direct child of this file path.  The child may or may not
+        exist.
+
+        @param name: the name of a child of this path. C{name} must be a direct
+            child of this path and may not contain a path separator.
+        @return: the child of this path with the given C{name}.
+        @raise InsecurePath: if C{name} describes a file path that is not a
+            direct child of this file path.
+        """
+
+    def open(mode="r"):
+        """
+        Opens this file path with the given mode.
+        @return: a file-like-object.
+        @raise Exception: if this file path cannot be opened.
+        """
+
+    def changed():
+        """
+        Clear any cached information about the state of this path on disk.
+        """
+
+    def getsize():
+        """
+        @return: the size of the file at this file path in bytes.
+        @raise Exception: if the size cannot be obtained.
+        """
+
+    def getModificationTime():
+        """
+        Retrieve the time of last access from this file.
+
+        @return: a number of seconds from the epoch.
+        @rtype: float
+        """
+
+    def getStatusChangeTime():
+        """
+        Retrieve the time of the last status change for this file.
+
+        @return: a number of seconds from the epoch.
+        @rtype: float
+        """
+
+    def getAccessTime():
+        """
+        Retrieve the time that this file was last accessed.
+
+        @return: a number of seconds from the epoch.
+        @rtype: float
+        """
+
+    def exists():
+        """
+        @return: C{True} if the file at this file path exists, C{False}
+            otherwise.
+        """
+
+    def isdir():
+        """
+        @return: C{True} if the file at this file path is a directory, C{False}
+            otherwise.
+        """
+
+    def isfile():
+        """
+        @return: C{True} if the file at this file path is a regular file,
+            C{False} otherwise.
+        """
+
+    def children():
+        """
+        @return: a sequence of the children of the directory at this file path.
+        @raise Exception: if the file at this file path is not a directory.
+        """
+
+    def basename():
+        """
+        @return: the base name of this file path.
+        """
+
+    def parent():
+        """
+        A file path for the directory containing the file at this file path.
+        """
+
+    def sibling(name):
+        """
+        A file path for the directory containing the file at this file path.
+        @param name: the name of a sibling of this path. C{name} must be a direct
+            sibling of this path and may not contain a path separator.
+
+        @return: a sibling file path of this one.
+        """
+
 class InsecurePath(Exception):
     """
     Error that is raised when the path provided to FilePath is invalid.
@@ -130,10 +265,13 @@ def _secureEnoughString():
 
 
 
-class _PathHelper:
+class AbstractFilePath(object):
     """
-    Abstract helper class also used by ZipPath; implements certain utility
-    methods.
+    Abstract implementation of an IFilePath; must be completed by a subclass.
+
+    This class primarily exists to provide common implementations of certain
+    methods in IFilePath. It is *not* a required parent class for IFilePath
+    implementations, just a useful starting point.
     """
 
     def getContent(self):
@@ -246,8 +384,36 @@ class _PathHelper:
 
 
     def sibling(self, path):
+        """
+        Return a L{FilePath} with the same directory as this instance but with a
+        basename of C{path}.
+
+        @param path: The basename of the L{FilePath} to return.
+        @type path: C{str}
+
+        @rtype: L{FilePath}
+        """
         return self.parent().child(path)
 
+
+    def descendant(self, segments):
+        """
+        Retrieve a child or child's child of this path.
+
+        @param segments: A sequence of path segments as C{str} instances.
+
+        @return: A L{FilePath} constructed by looking up the C{segments[0]}
+            child of this path, the C{segments[1]} child of that path, and so
+            on.
+
+        @since: 10.2
+        """
+        path = self
+        for name in segments:
+            path = path.child(name)
+        return path
+
+
     def segmentsFrom(self, ancestor):
         """
         Return a list of segments between a child and its ancestor.
@@ -310,7 +476,96 @@ class _PathHelper:
 
 
 
-class FilePath(_PathHelper):
+class RWX(FancyEqMixin, object):
+    """
+    A class representing read/write/execute permissions for a single user
+    category (i.e. user/owner, group, or other/world).  Instantiate with
+    three boolean values: readable? writable? executable?.
+
+    @type read: C{bool}
+    @ivar read: Whether permission to read is given
+
+    @type write: C{bool}
+    @ivar write: Whether permission to write is given
+
+    @type execute: C{bool}
+    @ivar execute: Whether permission to execute is given
+
+    @since: 11.1
+    """
+    compareAttributes = ('read', 'write', 'execute')
+    def __init__(self, readable, writable, executable):
+        self.read = readable
+        self.write = writable
+        self.execute = executable
+
+
+    def __repr__(self):
+        return "RWX(read=%s, write=%s, execute=%s)" % (
+            self.read, self.write, self.execute)
+
+
+    def shorthand(self):
+        """
+        Returns a short string representing the permission bits.  Looks like
+        part of what is printed by command line utilities such as 'ls -l'
+        (e.g. 'rwx')
+        """
+        returnval = ['r', 'w', 'x']
+        i = 0
+        for val in (self.read, self.write, self.execute):
+            if not val:
+                returnval[i] = '-'
+            i += 1
+        return ''.join(returnval)
+
+
+
+class Permissions(FancyEqMixin, object):
+    """
+    A class representing read/write/execute permissions.  Instantiate with any
+    portion of the file's mode that includes the permission bits.
+
+    @type user: L{RWX}
+    @ivar user: User/Owner permissions
+
+    @type group: L{RWX}
+    @ivar group: Group permissions
+
+    @type other: L{RWX}
+    @ivar other: Other/World permissions
+
+    @since: 11.1
+    """
+
+    compareAttributes = ('user', 'group', 'other')
+
+    def __init__(self, statModeInt):
+        self.user, self.group, self.other = (
+            [RWX(*[statModeInt & bit > 0 for bit in bitGroup]) for bitGroup in
+             [[S_IRUSR, S_IWUSR, S_IXUSR],
+              [S_IRGRP, S_IWGRP, S_IXGRP],
+              [S_IROTH, S_IWOTH, S_IXOTH]]]
+        )
+
+
+    def __repr__(self):
+        return "[%s | %s | %s]" % (
+            str(self.user), str(self.group), str(self.other))
+
+
+    def shorthand(self):
+        """
+        Returns a short string representing the permission bits.  Looks like
+        what is printed by command line utilities such as 'ls -l'
+        (e.g. 'rwx-wx--x')
+        """
+        return "".join(
+            [x.shorthand() for x in (self.user, self.group, self.other)])
+
+
+
+class FilePath(AbstractFilePath):
     """
     I am a path on the filesystem that only permits 'downwards' access.
 
@@ -356,31 +611,56 @@ class FilePath(_PathHelper):
     @type statinfo: C{int} or L{types.NoneType} or L{os.stat_result}
     """
 
+    implements(IFilePath)
+
     statinfo = None
     path = None
 
+    sep = slash
+
     def __init__(self, path, alwaysCreate=False):
+        """
+        Convert a path string to an absolute path if necessary and initialize
+        the L{FilePath} with the result.
+        """
         self.path = abspath(path)
         self.alwaysCreate = alwaysCreate
 
     def __getstate__(self):
+        """
+        Support serialization by discarding cached L{os.stat} results and
+        returning everything else.
+        """
         d = self.__dict__.copy()
         if d.has_key('statinfo'):
             del d['statinfo']
         return d
 
+
     def child(self, path):
+        """
+        Create and return a new L{FilePath} representing a path contained by
+        C{self}.
+
+        @param path: The base name of the new L{FilePath}.  If this contains
+            directory separators or parent references it will be rejected.
+        @type path: C{str}
+
+        @raise InsecurePath: If the result of combining this path with C{path}
+            would result in a path which is not a direct child of this path.
+        """
         if platform.isWindows() and path.count(":"):
             # Catch paths like C:blah that don't have a slash
             raise InsecurePath("%r contains a colon." % (path,))
         norm = normpath(path)
-        if slash in norm:
+        if self.sep in norm:
             raise InsecurePath("%r contains one or more directory separators" % (path,))
         newpath = abspath(joinpath(self.path, norm))
         if not newpath.startswith(self.path):
             raise InsecurePath("%r is not a child of %s" % (newpath, self.path))
         return self.clonePath(newpath)
 
+
     def preauthChild(self, path):
         """
         Use me if `path' might have slashes in it, but you know they're safe.
@@ -508,6 +788,9 @@ class FilePath(_PathHelper):
         @param reraise: a boolean.  If true, re-raise exceptions from
         L{os.stat}; otherwise, mark this path as not existing, and remove any
         cached stat information.
+
+        @raise Exception: is C{reraise} is C{True} and an exception occurs while
+            reloading metadata.
         """
         try:
             self.statinfo = stat(self.path)
@@ -588,9 +871,134 @@ class FilePath(_PathHelper):
         return float(st.st_atime)
 
 
+    def getInodeNumber(self):
+        """
+        Retrieve the file serial number, also called inode number, which 
+        distinguishes this file from all other files on the same device.
+
+        @raise: NotImplementedError if the platform is Windows, since the
+                inode number would be a dummy value for all files in Windows
+        @return: a number representing the file serial number
+        @rtype: C{long}
+        @since: 11.0
+        """
+        if platform.isWindows():
+            raise NotImplementedError
+
+        st = self.statinfo
+        if not st:
+            self.restat()
+            st = self.statinfo
+        return long(st.st_ino)
+
+
+    def getDevice(self):
+        """
+        Retrieves the device containing the file.  The inode number and device
+        number together uniquely identify the file, but the device number is
+        not necessarily consistent across reboots or system crashes.
+
+        @raise: NotImplementedError if the platform is Windows, since the
+                device number would be 0 for all partitions on a Windows
+                platform
+        @return: a number representing the device
+        @rtype: C{long}
+        @since: 11.0
+        """
+        if platform.isWindows():
+            raise NotImplementedError
+
+        st = self.statinfo
+        if not st:
+            self.restat()
+            st = self.statinfo
+        return long(st.st_dev)
+
+
+    def getNumberOfHardLinks(self):
+        """
+        Retrieves the number of hard links to the file.  This count keeps
+        track of how many directories have entries for this file.  If the
+        count is ever decremented to zero then the file itself is discarded
+        as soon as no process still holds it open.  Symbolic links are not
+        counted in the total.
+
+        @raise: NotImplementedError if the platform is Windows, since Windows
+                doesn't maintain a link count for directories, and os.stat
+                does not set st_nlink on Windows anyway.
+        @return: the number of hard links to the file
+        @rtype: C{int}
+        @since: 11.0
+        """
+        if platform.isWindows():
+            raise NotImplementedError
+
+        st = self.statinfo
+        if not st:
+            self.restat()
+            st = self.statinfo
+        return int(st.st_nlink)
+
+
+    def getUserID(self):
+        """
+        Returns the user ID of the file's owner.
+
+        @raise: NotImplementedError if the platform is Windows, since the UID
+                is always 0 on Windows
+        @return: the user ID of the file's owner
+        @rtype: C{int}
+        @since: 11.0
+        """
+        if platform.isWindows():
+            raise NotImplementedError
+
+        st = self.statinfo
+        if not st:
+            self.restat()
+            st = self.statinfo
+        return int(st.st_uid)
+
+
+    def getGroupID(self):
+        """
+        Returns the group ID of the file.
+
+        @raise: NotImplementedError if the platform is Windows, since the GID
+                is always 0 on windows
+        @return: the group ID of the file
+        @rtype: C{int}
+        @since: 11.0
+        """
+        if platform.isWindows():
+            raise NotImplementedError
+
+        st = self.statinfo
+        if not st:
+            self.restat()
+            st = self.statinfo
+        return int(st.st_gid)
+
+
+    def getPermissions(self):
+        """
+        Returns the permissions of the file.  Should also work on Windows,
+        however, those permissions may not what is expected in Windows.
+
+        @return: the permissions for the file
+        @rtype: L{Permissions}
+        @since: 11.1
+        """
+        st = self.statinfo
+        if not st:
+            self.restat()
+            st = self.statinfo
+        return Permissions(S_IMODE(st.st_mode))
+
+
     def exists(self):
         """
-        Check if the C{path} exists.
+        Check if this L{FilePath} exists.
 
         @return: C{True} if the stats of C{path} can be retrieved successfully,
             C{False} in the other cases.
@@ -607,6 +1015,10 @@ class FilePath(_PathHelper):
 
 
     def isdir(self):
+        """
+        @return: C{True} if this L{FilePath} refers to a directory, C{False}
+            otherwise.
+        """
         st = self.statinfo
         if not st:
             self.restat(False)
@@ -615,7 +1027,12 @@ class FilePath(_PathHelper):
                 return False
         return S_ISDIR(st.st_mode)
 
+
     def isfile(self):
+        """
+        @return: C{True} if this L{FilePath} points to a regular file (not a
+            directory, socket, named pipe, etc), C{False} otherwise.
+        """
         st = self.statinfo
         if not st:
             self.restat(False)
@@ -624,38 +1041,106 @@ class FilePath(_PathHelper):
                 return False
         return S_ISREG(st.st_mode)
 
+
+    def isBlockDevice(self):
+        """
+        Returns whether the underlying path is a block device.
+
+        @return: C{True} if it is a block device, C{False} otherwise 
+        @rtype: C{bool}
+        @since: 11.1
+        """
+        st = self.statinfo
+        if not st:
+            self.restat(False)
+            st = self.statinfo
+            if not st:
+                return False
+        return S_ISBLK(st.st_mode)
+
+
+    def isSocket(self):
+        """
+        Returns whether the underlying path is a socket.
+
+        @return: C{True} if it is a socket, C{False} otherwise 
+        @rtype: C{bool}
+        @since: 11.1
+        """
+        st = self.statinfo
+        if not st:
+            self.restat(False)
+            st = self.statinfo
+            if not st:
+                return False
+        return S_ISSOCK(st.st_mode)
+
+
     def islink(self):
+        """
+        @return: C{True} if this L{FilePath} points to a symbolic link.
+        """
         # We can't use cached stat results here, because that is the stat of
         # the destination - (see #1773) which in *every case* but this one is
         # the right thing to use.  We could call lstat here and use that, but
         # it seems unlikely we'd actually save any work that way.  -glyph
         return islink(self.path)
 
+
     def isabs(self):
+        """
+        @return: C{True}, always.
+        """
         return isabs(self.path)
 
+
     def listdir(self):
+        """
+        List the base names of the direct children of this L{FilePath}.
+
+        @return: a C{list} of C{str} giving the names of the contents of the
+            directory this L{FilePath} refers to.  These names are relative to
+            this L{FilePath}.
+
+        @raise: Anything the platform C{os.listdir} implementation might raise
+            (typically OSError).
+        """
         return listdir(self.path)
 
+
     def splitext(self):
+        """
+        @return: tuple where the first item is the filename and second item is
+            the file extension. See Python docs for C{os.path.splitext}
+        """
         return splitext(self.path)
 
+
     def __repr__(self):
         return 'FilePath(%r)' % (self.path,)
 
+
     def touch(self):
+        """
+        Updates the access and last modification times of the file at this
+        file path to the current time. Also creates the file if it does not
+        already exist.
+
+        @raise Exception: if unable to create or modify the last modification
+            time of the file.
+        """
         try:
             self.open('a').close()
         except IOError:
             pass
         utime(self.path, None)
 
+
     def remove(self):
         """
         Removes the file or directory that is represented by self.  If
         C{self.path} is a directory, recursively remove all its children
-        before removing the directory.  If it's a file or link, just delete
-        it.
+        before removing the directory. If it's a file or link, just delete it.
         """
         if self.isdir() and not self.islink():
             for child in self.children():
@@ -681,16 +1166,33 @@ class FilePath(_PathHelper):
         pattern.
         """
         import glob
-        path = self.path[-1] == '/' and self.path + pattern or slash.join([self.path, pattern])
+        path = self.path[-1] == '/' and self.path + pattern or self.sep.join([self.path, pattern])
         return map(self.clonePath, glob.glob(path))
 
+
     def basename(self):
+        """
+        @return: The final component of the L{FilePath}'s path (Everything after
+            the final path separator).
+        @rtype: C{str}
+        """
         return basename(self.path)
 
+
     def dirname(self):
+        """
+        @return: All of the components of the L{FilePath}'s path except the last
+            one (everything up to the final path separator).
+        @rtype: C{str}
+        """
         return dirname(self.path)
 
+
     def parent(self):
+        """
+        @return: A L{FilePath} representing the path which directly contains
+            this L{FilePath}.
+        """
         return self.clonePath(self.dirname())
 
 
@@ -759,14 +1261,25 @@ class FilePath(_PathHelper):
             return NotImplemented
         return cmp(self.path, other.path)
 
+
     def createDirectory(self):
+        """
+        Create the directory the L{FilePath} refers to.
+
+        @see: L{makedirs}
+
+        @raise OSError: If the directory cannot be created.
+        """
         os.mkdir(self.path)
 
+
     def requireCreate(self, val=1):
         self.alwaysCreate = val
 
+
     def create(self):
-        """Exclusively create a file, only if this file previously did not exist.
+        """
+        Exclusively create a file, only if this file previously did not exist.
         """
         fdint = os.open(self.path, _CREATE_FLAGS)
 
@@ -808,6 +1321,8 @@ class FilePath(_PathHelper):
         """
         Copies self to destination.
 
+        If self doesn't exist, an OSError is raised.
+
         If self is a directory, this method copies its children (but not
         itself) recursively to destination - if destination does not exist as a
         directory, this method creates it.  If destination is a file, an
@@ -871,6 +1386,8 @@ class FilePath(_PathHelper):
                     readfile.close()
             finally:
                 writefile.close()
+        elif not self.exists():
+            raise OSError(errno.ENOENT, "No such file or directory")
         else:
             # If you see the following message because you want to copy
             # symlinks, fifos, block devices, character devices, or unix
